package com.engine.organization.thread;

import com.engine.organization.entity.company.po.CompPO;
import com.engine.organization.entity.hrmresource.po.HrmResourcePO;
import com.engine.organization.entity.logview.bo.FieldBaseEquator;
import com.engine.organization.entity.map.JclOrgMap;
import com.engine.organization.enums.ModuleTypeEnum;
import com.engine.organization.mapper.comp.CompMapper;
import com.engine.organization.mapper.jclorgmap.JclOrgMapper;
import com.engine.organization.mapper.trigger.CompTriggerMapper;
import com.engine.organization.util.OrganizationDateUtil;
import com.engine.organization.util.db.MapperProxyFactory;
import org.apache.commons.collections.CollectionUtils;
import weaver.common.DateUtil;
import weaver.hrm.passwordprotection.domain.HrmResource;

import java.sql.Date;
import java.util.Calendar;
import java.util.List;

/**
 * @author:dxfeng
 * @createTime: 2022/08/30
 * @version: 1.0
 */
public class CompanyTriggerRunnable implements Runnable {

    private final CompPO oldCompany;
    private final CompPO newCompany;

    private CompTriggerMapper getCompTriggerMapper() {
        return MapperProxyFactory.getProxy(CompTriggerMapper.class);
    }

    public CompanyTriggerRunnable(CompPO oldCompany, CompPO newCompany) {
        this.oldCompany = oldCompany;
        this.newCompany = newCompany;
    }

    public CompanyTriggerRunnable(Long companyId) {
        this.oldCompany = new CompPO();
        this.newCompany = MapperProxyFactory.getProxy(CompMapper.class).listById(companyId);
    }

    public CompanyTriggerRunnable(CompPO newCompany) {
        this.oldCompany = new CompPO();
        this.newCompany = newCompany;
        this.newCompany.setDeleteType(1);
    }

    @Override
    public void run() {
        FieldBaseEquator fieldBaseEquator = new FieldBaseEquator();
        List<String> diffFields = fieldBaseEquator.getDiffFieldList(oldCompany, newCompany);
        if (CollectionUtils.isEmpty(diffFields)) {
            return;
        }
        // 判断
        if (diffFields.contains("compName") || diffFields.contains("compPrincipal") || diffFields.contains("parentCompany") || diffFields.contains("forbiddenTag") || diffFields.contains("deleteType")) {
            JclOrgMap jclMap = new JclOrgMap();

            jclMap.setFType(1);
            // 更新逻辑
            jclMap.setFObjId(newCompany.getId().intValue());
            jclMap.setId(newCompany.getId().intValue());
            jclMap.setUuid(newCompany.getUuid());
            jclMap.setFNumber(newCompany.getCompNo());
            jclMap.setFName(newCompany.getCompName());
            jclMap.setFLeader(newCompany.getCompPrincipal());
            jclMap.setFParentId(null == newCompany.getParentCompany() ? 0 : newCompany.getParentCompany().intValue());
            jclMap.setFObjParentId(null == newCompany.getParentCompany() ? 0 : newCompany.getParentCompany().intValue());
            jclMap.setFEcId(getCompTriggerMapper().getEcCompanyIdByUuid(jclMap.getUuid()));


            jclMap.setFClass(0);
            jclMap.setFClassName("行政维度");
            HrmResource hrmResourceById = getCompTriggerMapper().getHrmResourceById(jclMap.getFLeader());


            if (null != hrmResourceById) {
                jclMap.setFLeaderImg(hrmResourceById.getMessagerurl());
                jclMap.setFLeaderName(hrmResourceById.getLastname());
                jclMap.setFLeaderJobId(hrmResourceById.getJobtitle());
            }

            jclMap.setFLeaderJob(getCompTriggerMapper().getJobTitleMarkById(jclMap.getFLeaderJobId()));

            HrmResourcePO resourceByEcId = getCompTriggerMapper().getResourceByEcId(jclMap.getFLeader());
            if (null != resourceByEcId) {
                jclMap.setFLeaderSt(resourceByEcId.getJobGrade());
                jclMap.setFLeaderLv(resourceByEcId.getJobLevel());
            }
            String currentDate = OrganizationDateUtil.getFormatLocalDate(new java.util.Date());
            jclMap.setFDateBegin(new Date(OrganizationDateUtil.stringToDate(currentDate).getTime()));
            jclMap.setFDateEnd(new Date(OrganizationDateUtil.stringToDate("2099-12-31").getTime()));


            JclOrgMap jclOrgMapByObjID = MapperProxyFactory.getProxy(JclOrgMapper.class).getJclOrgMapByObjID(currentDate, ModuleTypeEnum.subcompanyfielddefined.getValue().toString(), jclMap.getFObjId().toString());
            if (null != jclOrgMapByObjID) {
                jclMap.setFPlan(jclOrgMapByObjID.getFPlan());
                jclMap.setFOnJob(jclOrgMapByObjID.getFOnJob());
            } else {
                jclMap.setFPlan(0);
                jclMap.setFOnJob(0);
            }
            jclMap.setFIsVitual(0);

            Calendar cal = Calendar.getInstance();
            cal.setTime(jclMap.getFDateBegin());
            Calendar calendar = DateUtil.addDay(cal, -1);
            Date time = new Date(calendar.getTime().getTime());
            getCompTriggerMapper().deleteMap(jclMap.getFType(), jclMap.getFObjId(), jclMap.getFDateBegin());
            getCompTriggerMapper().updateMap(jclMap.getFType(), jclMap.getFObjId(), jclMap.getFDateBegin(), time);


            if (1 != newCompany.getDeleteType() && 1 != newCompany.getForbiddenTag()) {
                MapperProxyFactory.getProxy(JclOrgMapper.class).insertMap(jclMap);
                if(null != jclOrgMapByObjID.getFParentId()){
                    updateParentPlanAndJob(currentDate,jclOrgMapByObjID.getFParentId().toString());
                }
                if(null!=oldCompany){
                    updateParentPlanAndJob(currentDate,oldCompany.getId().toString());
                }
            }
        }
    }

    /**
     * 更新上级部门在编、在岗数
     */
    void updateParentPlanAndJob(String currentDate, String parentId) {
        JclOrgMap parentJclOrgMap = MapperProxyFactory.getProxy(JclOrgMapper.class).getJclOrgMapById(currentDate, parentId);
        if (null != parentJclOrgMap) {
            // 上级部门当前在编、在岗数
            JclOrgMap jclOrgMapSum = MapperProxyFactory.getProxy(JclOrgMapper.class).getSumPlanAndJobByFParentId(currentDate, parentJclOrgMap.getId().toString());
            if (null != jclOrgMapSum) {
                parentJclOrgMap.setFPlan(jclOrgMapSum.getFPlan());
                parentJclOrgMap.setFOnJob(jclOrgMapSum.getFOnJob());
            } else {
                parentJclOrgMap.setFPlan(0);
                parentJclOrgMap.setFOnJob(0);
            }
            parentJclOrgMap.setFDateBegin(new Date(OrganizationDateUtil.stringToDate(currentDate).getTime()));
            parentJclOrgMap.setFDateEnd(new Date(OrganizationDateUtil.stringToDate("2099-12-31").getTime()));

            Calendar cal = Calendar.getInstance();
            cal.setTime(parentJclOrgMap.getFDateBegin());
            Calendar calendar = DateUtil.addDay(cal, -1);
            Date time = new Date(calendar.getTime().getTime());
            getCompTriggerMapper().deleteMap(parentJclOrgMap.getFType(), parentJclOrgMap.getFObjId(), parentJclOrgMap.getFDateBegin());
            getCompTriggerMapper().updateMap(parentJclOrgMap.getFType(), parentJclOrgMap.getFObjId(), parentJclOrgMap.getFDateBegin(), time);
            MapperProxyFactory.getProxy(JclOrgMapper.class).insertMap(parentJclOrgMap);
            if (null != parentJclOrgMap.getFParentId() && -1 != parentJclOrgMap.getFParentId()) {
                updateParentPlanAndJob(currentDate, parentJclOrgMap.getFParentId().toString());
            }
        }
    }
}
